==> spread-5.0.0.54-py3-none-any.whl: spread_core/bam/providers.py <==
import logging

import spread_core.mqtt as mqtt
from spread_core.bam.entities import Entity
from spread_core.tools import settings


class Provider(Entity):
    VALID = 'valid'
    SUSPICIOUS = 'suspicious'
    INVALID = 'invalid'

    def __init__(self, data):
        super(Provider, self).__init__(data['id'])
        self._manager_id = data['managerID']
        self._manager = None
        self._invalid_values = {}
        self._status = self.VALID
        self.name = data.get('name', f'id:{self.id}')

    @property
    def manager(self): return self._manager

    def get_address(self, funit_type):
        return mqtt.spread.address.ProviderAddress.of(self, funit_type)

    def publish_all(self, invalid):
        for funit_type, value in self.values.items():
            topic = mqtt.spread.topic.State(self.get_address(funit_type))
            variable = mqtt.spread.variable.Variable(value, invalid=invalid)
            self._manager.publish(topic, variable, retain=True)

    @property
    def surveyable(self):
        return True

    @property
    def infoble(self):
        return True

    @property
    def is_device(self):
        return True

    @property
    def is_group(self):
        return False

    @property
    def is_broadcast(self):
        return False

    @property
    def is_valid(self):
        return self._status != self.INVALID

    def set_manager(self, manager):
        self._manager = manager

    def send(self, cmd):
        return self._manager.send_interface.send(cmd)

    def get_bindings(self):
        pass

    def on_command_sended(self, cmd):
        pass

    def on_command_prepared(self, *commands, **kwargs):
        if kwargs.get('fake', False) is False:
            self.manager.add_command(self, *commands, **kwargs)

    def on_queue(self):
        pass

    def request(self, cmd, *args, **kwargs):
        raise NotImplementedError()

    def on_update(self, funit_type, value, sig=None):
        variable = mqtt.spread.variable.Variable(value)

        if value != self.values.get(funit_type, None):
            self.manager.publish(
                topic=mqtt.spread.topic.State(self.get_address(funit_type)),
                data=variable,
                retain=isinstance(funit_type, str) or funit_type.retained
            )
            if value is None:
                self._invalid_values.setdefault(funit_type, self.values.pop(funit_type, None))
            else:
                self.set_value(funit_type, value)
                self._invalid_values.pop(funit_type, None)

        if sig and sig.session_id and sig.funit_type == funit_type:
            if sig and sig.key:
                variable.key = sig.key
            self.manager.publish(mqtt.spread.topic.Reply(self.manager.project_id, sig.session_id), variable, retain=False)

    def on_request_answer(self, funit_type, response, sig=None):
        self.on_update(funit_type, None, sig=sig)

    def set_valid(self, validity):
        if self._status != validity:
            logging.info(f'{self} is {validity}')
            self._status = validity

    def on_survey(self):
        raise NotImplementedError()

    def get_info(self):
        raise NotImplementedError()

    def commit_states(self, is_valid):
        for funit_type, value in self._invalid_values.copy().items():
            if is_valid:
                self.on_state(funit_type, value)
            else:
                self.manager.publish(
                    mqtt.spread.topic.State(self.get_address(funit_type)),
                    mqtt.spread.variable.Variable(None),
                    retain=True
                )

        if not is_valid:
            for funit_type in self.repair_on_fail():
                self.manager.publish(
                    mqtt.spread.topic.State(self.get_address(funit_type)),
                    mqtt.spread.variable.Variable(None),
                    retain=True
                )

    def on_state(self, funit_type, value):
        self.on_update(funit_type, value)

    def check_valid(self):
        pass

    def on_exit(self):
        pass

    def override_info(self):
        for funit_type, value in settings.dump.get_dump_entity(self).items():
            self.on_update(funit_type, value)

    def get_funit(self, funit_type):
        if self._manager.__class__.__name__ in mqtt.classifier:
            if self.__class__.__name__ in mqtt.classifier[self._manager.__class__.__name__]:
                if funit_type is None:
                    return mqtt.classifier[self._manager.__class__.__name__][self.__class__.__name__]
                if funit_type in mqtt.classifier[self._manager.__class__.__name__][self.__class__.__name__]:
                    return mqtt.classifier[self._manager.__class__.__name__][self.__class__.__name__][funit_type]
        return None

==> spread-5.1.0.48-py3-none-any.whl: spread_core/bam/providers.py <==
import logging

import spread_core.mqtt as mqtt
from spread_core.bam.entities import Entity
from spread_core.tools import settings


class Provider(Entity):
    VALID = 'valid'
    SUSPICIOUS = 'suspicious'
    INVALID = 'invalid'

    def __init__(self, data):
        super(Provider, self).__init__(data['id'])
        self._manager_id = data['managerID']
        self._manager = None
        self._status = self.VALID
        self.name = data.get('name', f'id:{self.id}')

    @property
    def manager(self): return self._manager

    @property
    def project_id(self): return self._manager.project_id

    def publish(self, topic, data, retain=False, **kwargs):
        self._manager.publish(topic, data, retain, **kwargs)

    def get_address(self, funit_type):
        return mqtt.spread.address.ProviderAddress.of(self, funit_type)

    def publish_all(self, invalid):
        for funit_type, value in self.values.items():
            topic = mqtt.spread.topic.State(self.get_address(funit_type))
            variable = mqtt.spread.variable.Variable(value, invalid=invalid)
            self.publish(topic, variable, retain=True)

    @property
    def surveyable(self):
        return True

    @property
    def infoble(self):
        return True

    @property
    def is_device(self):
        return True

    @property
    def is_group(self):
        return False

    @property
    def is_broadcast(self):
        return False

    @property
    def is_valid(self):
        return self._status != self.INVALID

    def set_manager(self, manager):
        self._manager = manager

    def send(self, cmd):
        return self._manager.send_interface.send(cmd)

    def get_bindings(self):
        pass

    def on_command_sended(self, cmd):
        pass

    def on_command_prepared(self, *commands, **kwargs):
        if kwargs.get('fake', False) is False:
            self.manager.add_command(self, *commands, **kwargs)

    def on_queue(self):
        pass

    def request(self, cmd, *args, **kwargs):
        raise NotImplementedError()

    def on_request_answer(self, funit_type, response, sig=None, *args):
        self.on_update(funit_type, None, sig=sig)

    def set_valid(self, validity):
        if self._status != validity:
            logging.info(f'{self} is {validity}')
            self._status = validity

    def on_survey(self):
        raise NotImplemented()

    def get_info(self):
        raise NotImplemented()

    def commit_states(self, is_valid):
        for funit_type, value in self._invalid_values.copy().items():
            if is_valid:
                self.on_state(funit_type, value)
            else:
                self.publish(
                    mqtt.spread.topic.State(self.get_address(funit_type)),
                    mqtt.spread.variable.Variable(None),
                    retain=True
                )

        if not is_valid:
            for funit_type in self.repair_on_fail():
                self.publish(
                    mqtt.spread.topic.State(self.get_address(funit_type)),
                    mqtt.spread.variable.Variable(None),
                    retain=True
                )

    def on_state(self, funit_type, value):
        self.on_update(funit_type, value)

    def check_valid(self):
        pass

    def on_exit(self):
        self.publish_all(True)

    def override_info(self):
        for funit_type, value in settings.dump.get_dump_entity(self).items():
            self.on_update(funit_type, value)

    def get_funit(self, funit_type):
        if self._manager.__class__.__name__ in mqtt.classifier:
            if self.__class__.__name__ in mqtt.classifier[self._manager.__class__.__name__]:
                if funit_type is None:
                    return mqtt.classifier[self._manager.__class__.__name__][self.__class__.__name__]
                if funit_type in mqtt.classifier[self._manager.__class__.__name__][self.__class__.__name__]:
                    return mqtt.classifier[self._manager.__class__.__name__][self.__class__.__name__][funit_type]
        return None
